<?php
/**
 * Copyright since 2022 Inferendo Srl
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to hello@visidea.ai so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Visidea to newer
 * versions in the future. If you wish to customize Visidea for your
 * needs please refer to https://visidea.ai/ for more information.
 *
 * @author    Inferendo SRL <hello@visidea.ai>
 * @copyright Since 2022 Inferendo Srl
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 */

class visideaCronModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        parent::initContent();

        $this->generateCsv();

    }

    private function generateCsv()
    {
        $cronKey = Configuration::get('VISIDEA_PRIVATE_KEY');

        $urlVar = $_GET;
        if (!isset($urlVar['token']) || !$urlVar['token'] || $cronKey != $urlVar['token'])
        {
            echo 'Token error!';
            die();
        }

        $path = __DIR__ . "/../../csv/";
        if (!is_dir($path)) {
            mkdir($path, 0777, true);
        }

        // items file
        $this->exportItems($path, $urlVar['token']);

        // // interactions file
        $this->exportInteractions($path, $urlVar['token']);

        // // users file
        $users = $this->exportUsers($path, $urlVar['token']);

        echo 'Files successfully exported!';
        die();
    }


    public function sanitizeString($string) {
        $sanitized = preg_replace('#<[^>]+>#', ' ', $string);
        $sanitized = str_replace('"', '\"', $sanitized);
        return $sanitized;
    }


    private function getAdditionalLanguages($languages, $default_language_id) {
        $additionalLanguages = [];
        foreach ($languages as $language) {
          if ($language['id_lang'] != $default_language_id)
              $additionalLanguages[] = $language;
        }
        return $additionalLanguages;
      }
      
    private function exportItems($path, $token)
    {

        $default_language_id = (int)Configuration::get('PS_LANG_DEFAULT');
        $languages = $this->getAdditionalLanguages(Language::getLanguages(true, $this->context->shop->id), $default_language_id);

        $filename = 'items_' . $token . '.csv';
        $temp_filename = 'items_' . $token . '.tmp';
        $hash_filename = 'items_' . $token . '.hash';

        $output = fopen($path . $temp_filename, "w");
        $header = "item_id;code;mpn;ean;name;brand_id;brand_name;price;market_price;discount;page_ids;page_names;url;images;stock;description;";
        foreach ($languages as $language) {
            $header .= 'name_'.$language['language_code'].';';
            $header .= 'description_'.$language['language_code'].';';
            $header .= 'page_names_'.$language['language_code'].';';
            $header .= 'url_'.$language['language_code'].';';
        }
        $header .= "\n";
        fputs($output, $header);

        $sql = 'SELECT `id_product` FROM `' . _DB_PREFIX_ . 'product`';
        $productsId = Db::getInstance()->executeS($sql);

        foreach ($productsId as $productId)
        {
            $newProduct = array();
            $newProduct['item_id'] = $productId['id_product'];

            $prod = new Product($newProduct['item_id']);

            $newProduct['code'] = $prod->reference;
            $eans = array();
            if ($prod->ean13 != '')
                $eans[] = $prod->ean13;
            if ($prod->isbn != '')
                $eans[] = $prod->isbn;
            if ($prod->upc != '')
                $eans[] = $prod->upc;
            $mpns = array();
            if ($prod->mpn != '')
                $mpns[] = $prod->mpn;
            // Fetch combinations for the current product
            $combinations = $prod->getAttributeCombinations((int)Context::getContext()->language->id);
            foreach ($combinations as $combination) {
                if (!empty($combination['ean13']))
                    $eans[] = $combination['ean13'];
                if (!empty($combination['isbn']))
                    $eans[] = $combination['isbn'];
                if (!empty($combination['upc']))
                    $eans[] = $combination['upc'];
                if (!empty($combination['mpn']))
                    $mpns[] = $combination['mpn'];
            }
            $newProduct['mpn'] = implode('|',$mpns);
            $newProduct['ean'] = implode('|',$eans);
            $newProduct['name'] = $this->sanitizeString($prod->name[$default_language_id]);
            $newProduct['brand_id'] = $prod->id_manufacturer;
            $newProduct['brand_name'] = $this->sanitizeString(Manufacturer::getNameById((int)$newProduct['brand_id']));
            $newProduct['price'] = $prod->getPriceWithoutReduct();
            $newProduct['market_price'] = $prod->getPrice();
            if ($newProduct['price'] > 0)
                $newProduct['discount'] = abs(($newProduct['market_price'] - $newProduct['price']) / $newProduct['price'] * 100);
            else
                $newProduct['discount'] = 0;

            $categories = Product::getProductCategoriesFull($newProduct['item_id'], $default_language_id);
            $catIds = array();
            $catNames = array();
            foreach ($categories as $cat) {
                $catIds[] = $cat['id_category'];
                $catNames[] = $this->sanitizeString($cat['name']);
            }
            $newProduct['page_ids'] = implode('|', $catIds);
            $newProduct['page_names'] = implode('|', $catNames);

            $link = new Link();
            $newProduct['url'] = $link->getProductLink($prod,null,null,null,$default_language_id);

            $productImages = $prod->getImages($default_language_id);
            $imagePathArr = array();
            foreach ($productImages AS $key => $val)
            {
                $id_image = $val['id_image'];
                $imagePath = "http://".$link->getImageLink($prod->link_rewrite[Context::getContext()
                    ->language->id], $id_image, 'home_default');
                $imagePathArr[] = $imagePath;
            }
            $newProduct['images'] = implode('|', $imagePathArr);

            $newProduct['stock'] = Product::getQuantity($newProduct['item_id']);

            $newProduct['description'] = $this->sanitizeString($prod->description[$default_language_id]);

            foreach ($languages as $language) {
                $newProduct['name_'.$language['language_code']] = $this->sanitizeString($prod->name[$language['id_lang']]);
                $newProduct['description_'.$language['language_code']] = $this->sanitizeString($prod->description[$language['id_lang']]);

                $categories = Product::getProductCategoriesFull($newProduct['item_id'], $language['id_lang']);
                $catNames = array();
                foreach ($categories as $cat) {
                    $catNames[] = $this->sanitizeString($cat['name']);
                }
                $newProduct['page_names_'.$language['language_code']] = implode('|', $catNames);

                $link = new Link();
                // $newProduct['url'] = $link->getProductLink($prod);
                $newProduct['url_'.$language['language_code']] = $link->getProductLink($prod,null,null,null,$language['id_lang']);
            }
    
            fputcsv($output, $newProduct, ";", "\"", "\\");
        }

        fclose($output);

        if (file_exists($path . $filename)) {
            unlink($path . $filename);
        }
        rename($path . $temp_filename, $path . $filename);

        file_put_contents($path  . $hash_filename, hash_file('md5',$path  . $filename));

    }

    private function exportUsers($path, $token)
    {

        $filename = 'users_' . $token . '.csv';
        $temp_filename = 'users_' . $token . '.tmp';
        $hash_filename = 'users_' . $token . '.hash';

        $output = fopen($path . $temp_filename, "w");
        fputs($output, "user_id;email;name;surname;address;city;zip;state;country;birthday\n");

        $sql = 'SELECT `id_customer` FROM `' . _DB_PREFIX_ . 'customer`';
        $customersId = Db::getInstance()->executeS($sql);

        foreach ($customersId as $customerId)
        {
            $newCustomer = array();
            $newCustomer['user_id'] = $customerId['id_customer'];

            $cust = new Customer((int)$newCustomer['user_id']);

            $newCustomer['email'] = $cust->email;
            $newCustomer['name'] = $cust->firstname;
            $newCustomer['surname'] = $cust->lastname;

            $addressId = Address::getFirstCustomerAddressId($newCustomer['user_id']);
            $address = new Address($addressId);

            $newCustomer['address'] = $address->address1;
            $newCustomer['city'] = $address->city;
            $newCustomer['zip'] = $address->postcode;

            if ($address->id_state > 0 && $address->id_country > 0) {
                $sql = 'SELECT `name` FROM `' . _DB_PREFIX_ . 'state` WHERE `id_state` = ' . $address->id_state . ' AND `id_country` = ' . $address->id_country;
                $state = Db::getInstance()->getRow($sql);
                $newCustomer['state'] = $state['name'];
            } else {
                $newCustomer['state'] = '';
            }

            if ($address->id_country > 0) {
                $sql = 'SELECT `iso_code` FROM `' . _DB_PREFIX_ . 'country` WHERE `id_country` = ' . $address->id_country;
                $country = Db::getInstance()->getRow($sql);
                $newCustomer['country'] = strtolower($country['iso_code']);
            } else {
                $newCustomer['country'] = '';
            }

            $newCustomer['birthday'] = $cust->birthday;

            fputcsv($output, $newCustomer, ";", "\"", "\\");
        }

        fclose($output);

        if (file_exists($path . $filename)) {
            unlink($path . $filename);
        }
        rename($path . $temp_filename, $path . $filename);

        file_put_contents($path  . $hash_filename, hash_file('md5',$path  . $filename));

    }

    private function exportInteractions($path, $token)
    {

        $filename = 'interactions_' . $token . '.csv';
        $temp_filename = 'interactions_' . $token . '.tmp';
        $hash_filename = 'interactions_' . $token . '.hash';

        $output = fopen($path . $temp_filename, "w");
        fputs($output, "user_id;item_id;action;timestamp\n");

        $sql = 'SELECT * FROM `' . _DB_PREFIX_ . 'cart_product` INNER JOIN ' . _DB_PREFIX_ . 'cart ON ' . _DB_PREFIX_ . 'cart_product.id_cart=' . _DB_PREFIX_ . 'cart.id_cart';
        $result = Db::getInstance()->executeS($sql);

        $oSql = 'SELECT `id_order`,`id_cart` FROM `' . _DB_PREFIX_ . 'orders`';
        $orderList = Db::getInstance()->executeS($oSql);

        $oItm = array();
        foreach ($orderList as $oderItm)
        {
            $oItm[$oderItm['id_order']] = $oderItm['id_cart'];
        }

        $cartProduct = array();
        foreach ($result as $product)
        {
            if ($product['id_customer']) $cartProduct['user_id'] = $product['id_customer'];
            else $cartProduct['user_id'] = $product['id_guest'];
            $cartProduct['item_id'] = $product['id_product'];
            if (in_array($product['id_cart'], $oItm)) $cartProduct['action'] = 'purchase';
            else $cartProduct['action'] = 'cart';

            $timestamp = new DateTime($product['date_upd']);
            $cartProduct['timestamp'] = $timestamp->format(DateTime::ISO8601);

            if ($cartProduct['user_id'] > 0 && $cartProduct['item_id'] > 0)
                fputcsv($output, $cartProduct, ";", "\"", "\\");
        }

        fclose($output);

        if (file_exists($path . $filename)) {
            unlink($path . $filename);
        }
        rename($path . $temp_filename, $path . $filename);

        file_put_contents($path  . $hash_filename, hash_file('md5',$path  . $filename));

    }

}
